﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Termination {
  [Item("MaximumIterationsTerminator", "")]
  [StorableClass]
  public class MaximumIterationsTerminator : Terminator, IThresholdTerminator/*, IIterationBasedOperator*/ {
    public ILookupParameter<IntValue> IterationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Iterations"]; }
    }

    [Storable]
    private IFixedValueParameter<IntValue> maximumIterationsParameter;
    public IFixedValueParameter<IntValue> MaximumIterationsParameter {
      get { return maximumIterationsParameter; }
      set {
        if (value == null) throw new ArgumentNullException("MaximumIterations parameter must not be null.");
        if (value.Value == null) throw new ArgumentNullException("MaximumIterations parameter value must not be null.");
        if (maximumIterationsParameter == value) return;

        if (maximumIterationsParameter != null) Parameters.Remove(maximumIterationsParameter);
        maximumIterationsParameter = value;
        Parameters.Add(maximumIterationsParameter);
        OnMaximumIterationsParameterChanged();
      }
    }
    IParameter IThresholdTerminator.ThresholdParameter {
      get { return MaximumIterationsParameter; }
    }

    public int MaximumIterations {
      get { return MaximumIterationsParameter.Value.Value; }
      set { MaximumIterationsParameter.Value.Value = value; }
    }


    [StorableConstructor]
    protected MaximumIterationsTerminator(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      Initialize();
    }
    protected MaximumIterationsTerminator(MaximumIterationsTerminator original, Cloner cloner)
      : base(original, cloner) {
      Initialize();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MaximumIterationsTerminator(this, cloner);
    }

    public MaximumIterationsTerminator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("Iterations", "The current iteration.") { Hidden = false });
      Parameters.Add(new FixedValueParameter<IntValue>("MaximumIterations", "The maximum number of iterations the algorithm should perform."));
      Initialize();
    }

    private void Initialize() {
      IterationsParameter.ActualNameChanged += new EventHandler(IterationsParameter_ActualNameChanged);
      RegisterMaximumIterationsParameterEvents();
    }

    private void MaximumIterations_ValueChanged(object sender, EventArgs e) {
      OnMaximumIterationsChanged();
    }
    private void IterationsParameter_ActualNameChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }

    protected virtual void OnMaximumIterationsChanged() {
      OnToStringChanged();
    }
    private void OnMaximumIterationsParameterChanged() {
      RegisterMaximumIterationsParameterEvents();
    }

    private void RegisterMaximumIterationsParameterEvents() {
      MaximumIterationsParameter.Value.ValueChanged += new EventHandler(MaximumIterations_ValueChanged);
    }

    protected override bool CheckContinueCriterion() {
      int iterations = IterationsParameter.ActualValue.Value;

      return iterations < MaximumIterations;
    }

    public override string ToString() {
      return string.Format("{0} < {1}", IterationsParameter.ActualName, MaximumIterations);
    }
  }
}